package com.palserv.component;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

/**
 * The DateBox class is a widget for displaying, editing, and
 * user entry of a date.  The format of the date is configurable
 * at runtime using a subclass of DateFormat (the default is
 * DFHyphen).  By using a different subclass in the constructor,
 * a different format of date will be allowed.
 * <p>
 * This class implements the CallPalCal interface so that
 * it can use a PalCal object to display the date as a
 * "calendar".
 */
public class DateBox extends JPanel implements CallPalCal {
	String useDate;
	PalCal palCal;
	JTextField jtf;
	String dfName;
	DateFormatter df;

	DateBox(boolean isCalledFromMain) {
		if (!isCalledFromMain) {
			add(makeContentPane());
		}
	}

/**
 * Default constructor defaults to YYYY-MM-DD format.
 */
	public DateBox() {
		this(false);
	}

/**
 * Constructs a DateBox with the specified date format.
 * @param dfs a fully qualified name of a subclass of DateFormat
 */
	public DateBox(String dfs) {
		dfName = dfs;
// DateFormatter.spt("DateBox.constructor: "+dfs);
		add(makeContentPane());
	}

	DateBox(boolean isCalledFromMain, String dfs) {
		dfName = dfs;
		if (!isCalledFromMain) {
			add(makeContentPane());
		}
	}

/**
 * used for testing
 */
	public static void main(String[] args) {

		JFrame aFrame = new JFrame("DateBox");
		aFrame.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			}
		);
		String dfs = "";
		String ud = "";
		for (int i = 0; i < args.length; i ++) {
			if (args[i].substring(0,1).equals("-")) {
				if (args[i].substring(1,2).equals("f")) {
					dfs = args[i].substring(2);
				}
				if (args[i].substring(1,2).equals("u")) {
					ud = args[i].substring(2);
				}
			}
		}

		DateBox pc = new DateBox(true, dfs);
		pc.useDate = ud;
//		  pc.getArgs(args);
		Container apane = pc.makeContentPane();
		aFrame.setContentPane(apane);

//		aFrame.setSize(149,233);
		aFrame.pack();
		aFrame.setVisible(true);
	}

/**
 * Get the text from the JTextField.
 * @return the text from the JTextField.
 */
	public String getText() {
		return jtf.getText();
	}

/**
 * Set the text of the JTextField.
 * @param the text to use - should be a valid date in the current format.
 */
	public void setText(String s) {
		jtf.setText(s);
	}

/**
 * get the ImageIcon for an image file
 */
	ImageIcon makeGif(String gName) {
		String iName = "/images/" + gName;
		return new ImageIcon(getClass().getResource(iName));
	}

/**
 * Build the main JPanel to display
 */
	JPanel makeContentPane() {
		try {
			UIManager.setLookAndFeel(
				UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			palCal.rpt("Unable to set system laf: "+e.getMessage());
		}
		if (dfName == null) {
			dfName = "com.palserv.component.DFHyphen";
		}
		if (df == null) {
			df = DateFormatter.DFFactory(dfName);
			if (df == null) {
				DateFormatter.rpt("DateBox.makeContentPane: null df");
			}
		}
		JPanel mainBox = new JPanel();
		FlowLayout fl = new FlowLayout();
		fl.setHgap(0);
		fl.setVgap(0);
		mainBox.setLayout(fl);
		jtf = new JTextField(15);
		jtf.setText(useDate);
		jtf.setBorder(null);
		mainBox.add(jtf);
		ImageIcon arrow = makeGif("downButArrow.gif");
		JButton jb = new JButton();
		jb.setFocusPainted(false);
		jb.setIcon(arrow);
		jb.setBorder(new BevelBorder(BevelBorder.RAISED));
		mainBox.setBorder(new BevelBorder(BevelBorder.LOWERED));
		mainBox.add(jb);
		jb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				openPalCal();
			}
		});

		return mainBox;
	}

/**
 * Get the current date in the current date format
 * @return current date in current date format
 */
	String getCurrDate() {
		GregorianCalendar c = new GregorianCalendar();
		int m = c.get(Calendar.MONTH) + 1;
		int d = c.get(Calendar.DATE);
		int y = c.get(Calendar.YEAR);
		// get current date
//		  useDate = "7/28/01";
//		  return ""+m+"/"+d+"/"+y;
		return df.getDateString(m, d, y);
	}

// methods to implement CallPalCal **********************************
/**
 * Part of the CallPalCal interface.  Used internally by the
 * PalCal class to get the date to display in the "calendar" display.
 * @return the date to display
 */
	public String getUseDate() {
		if (useDate == null || useDate.equals("")) {
			String ud = jtf.getText();
			if (ud == null || ud.equals("")) {
				useDate = getCurrDate();
// System.out.println("DateBox.getUseDate: got from system: useDate = "+useDate);
			} else {
				useDate = ud;
			}
		}
// System.out.println("DateBox.getUseDate: useDate = "+useDate);
		return useDate;
	}

/**
 * Part of the CallPalCal interface.  Used internally by the
 * PalCal class to set the date to display in the JTextField.
 * @param ud date "returned" by PalCal
 */
	public void setResponse(String ud) {
		useDate = ud;
		palCal.dispose();
		palCal = null;
		jtf.setText(useDate);
	}

/**
 * Part of the CallPalCal interface.  Used internally by the
 * PalCal class to set the current DateFormatter object.
 * The DateFormatter object should be the same one used
 * by this class.  There is no reason not to share this
 * object.	But if (for some reason unknown to me) someone
 * decides to use a different object, then
 * it is critical that the DateFormatter object
 * returned by this method be an instance of the same
 * class used to instantiate the one used by this class.
 * If they don't match up, then there is a possibility
 * that this class and PalCal will be using different
 * date formats, which will almost certainly lead to
 * trouble.
 * @return used to set date format
 */
	public DateFormatter getDF() {
		return df;
	}
// end of methods to implement CallPalCal **********************************

/**
 * Invoked when user hits the little "arrow" button on
 * the right side of the JTextField.  This tells us that
 * the user wants to "open" the DateBox by displaying
 * a calendar representation (a PalCal object).
 */
	void openPalCal() {
		String ud = jtf.getText();
		if (ud == null || ud.equals("")) {
			if (useDate == null || useDate.equals("")) {
				useDate = getCurrDate();
			}
		} else {
			int[] id = df.parseDate(ud);
			if (id[0] == -1) {
				jtf.setText(ud+" invalid");
				return;
			}
/*
			int m = id[0] + 1;
			useDate = ""+m+"/"+id[1]+"/"+id[2];
*/
			useDate = df.getDateString(id[0]+1, id[1], id[2]);
		}
		if (palCal == null) {
			palCal = new PalCal(this);
		}
		palCal.setContentPane(palCal.makeContentPane());
		int jtfX = jtf.getX();
		int jtfY = jtf.getY();

		// try to get set the location of the PalCal object
		Container owner = getRootOwner();
// palCal.spt("DateBox.openPalCal: owner = "+owner.getClass().getName());
		int ownX = owner.getX();
		int ownY = owner.getY();

// palCal.spt("DateBox.openPalCal: jft location = "+jtfX+", "+jtfY);
// palCal.spt("DateBox.openPalCal: this location = "+thisX+", "+thisY);
// palCal.spt("DateBox.openPalCal: owner location = "+ownX+", "+ownY);
		palCal.setLocation(jtfX+ownX+jtf.getHeight(), jtfY+ownY);
		palCal.pack();
		palCal.setVisible(true);
//		  palCal.spt("button hit");
//palCal.spt("DateBox.openPalCal: after constructor; useDate = "+useDate);
	}

/**
 * used by openPalCal method to find the "earliest" Container
 * in the parent chain.  This object will (hopefully) have
 * a useable location.
 */
	Container getRootOwner() {
		boolean noOwner = true;
		Container o = this;
		while (noOwner) {
			Container b = o.getParent();

			if (b == null) {
				noOwner = false;
			} else {
				Class c = b.getClass();

// palCal.spt("DateBox.getRootOwner: "+c.getName());
				o = b;
			}
		}
		return o;
	}
}
