package com.palserv.component;

import java.lang.reflect.*;

/**
 * The DateFormatter class encapsulates a date format for
 * use by com.palserv.component.DateBox, although it
 * is not in any way tied to DateBox, and could easily
 * be used independently.
 * <p>
 * DateFormatter is the base (abstract) class.
 * Client code should always refer to subclasses
 * by the base name.
 * <p>
 * The DFFactory (static) method can be used to create an instance of
 * a subclass of DateFactory.  It requires the fully
 * qualified name of the subclass.
 * As an example:
 * <pre>
 *	   DateFormatter df = DateFormatter.DFFactory("com.palserv.component.DFSlash");
 * </pre>
 * After this instruction, the variable df will contain an instance
 * of the DFSlash class.
 */
abstract class DateFormatter {

/**
 * Convert three int's to a date String.
 * This is the "opposite" of the parseDate method.
 * Subclasses should implement this according to the particular
 * format that they represent.
 * @param m month (0 = January, 11 = December)
 * @param d day (1 - 31)
 * @param y year
 * @return date formatted as String according to subclass
 */
	abstract String getDateString(int m, int d, int y);

/**
 * Convert a String to an array of three int's.
 * This is the "opposite" of the getDateString method.
 * The array of three int's should be in the order:
 * month, day, year.  This should be true regardless of
 * the subclass or the specific format.
 * @param ud date formatted according to subclass.
 * @return array of three int's: month, day, year
 */
	abstract int[] parseDate(String ud);

/**
 * Returns an object which "is a" (extends) DateFormatter.
 * The name passed must be the fully qualified name of a
 * class which extends DateFormatter.
 * @param cNam fully qualified name of class which extends DateFormatter
 * @return date format object
 */
	static DateFormatter DFFactory(String cNam) {
		Class cClas = null;
		Constructor cConst = null;
		DateFormatter df = null;
		try {
			cClas = Class.forName(cNam);
		} catch(ClassNotFoundException e) {
			ept(e, "DateFormatter: class not found: "+cNam);
		}
		try {
			cConst = cClas.getConstructor(new Class[]{});
		} catch(NoSuchMethodException e) {
			ept(e, "DateFormatter: no default constructor: "+cNam);
		} catch (SecurityException e) {
			ept(e,"DateFormatter: security: "+cNam);
		}
		try {
			df = (DateFormatter)cConst.newInstance(new Object[]{});
		} catch (Exception e) {
			ept(e,"DateFormatter: Error loading DateFormatter: "+cNam);
		}
		return df;
	}

/**
 * Utility routine to parse a String representation of a date
 * into an array of int's.  The general format of the String
 * is: first number - delimiter - second number - delimiter
 * - third number.	Examples: 1975-11-21, 3/17/2000.
 * <p>
 * The meaning of the three numbers is subclass specific.
 * @param ud String to parse
 * @param delim delimiter
 * @return array of three int's
 */
	static int[] parseWithDelim(String ud, String delim) {
		int[] id = new int[3];
		int cnt = 0;
		int pos = 0;
		while (cnt < 3) {
			int p = ud.indexOf(delim, pos);
			if (p == -1) {
				String s = ud.substring(pos);
				id[cnt] = getIntFromString(s);
//				  id[cnt] = new Integer(s).intValue();
				break;
			} else {
				String s = ud.substring(pos, p);
				id[cnt] = getIntFromString(s);
//				  id[cnt] = new Integer(s).intValue();
				pos = p + 1;
			}
			if (id[cnt] == -1) {
			   rpt("parseWithDelim: invalid number: "+ud);
			}
			cnt ++;
		}
		if (cnt > 2) {
			rpt("parseWithDelim: too many fields: "+ud);
		}
		if (cnt < 2) {
			rpt("parseWithDelim: not enough fields: "+ud);
		}
		return id;
	}

/**
 * used by parseWithDelim to convert a string to a number.
 * @param s String to be converted to an int
 * @return int which corresponds to the String or -1 if an error
 */
	static int getIntFromString(String s) {
		int id = -1;
		try {
			id = new Integer(s).intValue();
		} catch(NumberFormatException e) {
			return id;
		}
		return id;
	}

/**
 * utility routine
 */
	static void ept(Exception e, String s) {
		System.err.println("Exception: "+s);
		System.err.println("Exception message: "+e.getMessage());
		System.exit(0);
	}

/**
 * utility routine
 */
	static void rpt(String s) {
		System.out.println(s);
		System.exit(0);
	}

/**
 * utility routine
 */
	static void spt(String s) {
		System.out.println(s);
	}
}

/**
 * This subclass of DateFormatter implements the most common date
 * format in the US, mm/dd/yyyy.
 */
class DFSlash extends DateFormatter {
	public DFSlash() {}

	String getDateString(int m, int d, int y) {
		return ""+m+"/"+d+"/"+y;
	}

	int[] parseDate(String ud) {
		int[] id = parseWithDelim(ud, "/");
		if (id[0] < 1 | id[0] > 12) {
			rpt("parseDate: invalid month: "+ud);
		}
		id[0] --;
		if (id[1] < 1 | id[1] > 31) {
			rpt("parseDate: invalid day: "+ud);
		}
		if (id[2] < 100) {
			if (id[2] < 75) {
				id[2] += 2000;
			} else {
				id[2] += 1900;
			}
		}
		if (id[2] < 1900) {
			rpt("parseDate: invalid year: "+ud);
		}
		return id;
	}
}

/**
 * This subclass of DateFormatter implements a fairly common date
 * format in the US, yyyy-mm-dd (sort order).
 */
class DFHyphen extends DateFormatter {
	public DFHyphen() {}

	String getDateString(int m, int d, int y) {
		return ""+y+"-"+m+"-"+d;
	}

	int[] parseDate(String ud) {
// spt("DFHyphen.parseDate: parsing: "+ud);
		int[] id = parseWithDelim(ud, "-");
		if (id[1] < 1 | id[1] > 12) {
			rpt("parseDate: invalid month: "+ud);
		}
		id[1] --;
		if (id[2] < 1 | id[2] > 31) {
			rpt("parseDate: invalid day: "+ud);
		}
		if (id[0] < 100) {
			if (id[0] < 75) {
				id[0] += 2000;
			} else {
				id[0] += 1900;
			}
		}
		if (id[0] < 1900) {
			rpt("parseDate: invalid year: "+ud);
		}
		int ix[] = new int[3];
		ix[0] = id[1];	// month
		ix[1] = id[2];	// day
		ix[2] = id[0];	// year
// spt("DFHyphen.parseDate: returning: "+ix[0]+","+ix[1]+","+ix[2]);
		return ix;
	}
}
